package kafka.example;

import kafka.example.config.SoftKafkaAvroDeserializer;
import org.apache.avro.generic.GenericRecord;
import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.config.SaslConfigs;
import org.apache.kafka.common.config.SslConfigs;
import org.apache.kafka.common.serialization.StringDeserializer;


import java.time.Duration;
import java.util.Collections;
import java.util.Properties;

public class Consumer {

    public static void startConsumer() {
        Properties properties = new Properties();

        // Credenciales de ejemplo.
        // Cámbialas por el usuario y contraseña que te ha proporcionado Soporte.
        String groupId = "test_change_me1";
        String username = "username";
        String password = "password";
        String topicName = "test-topic";
        String brokers = "public-broker1.nautilus.app:8096,public-broker2.nautilus.app:8096,public-broker3.nautilus.app:8096";

        properties.put(SaslConfigs.SASL_JAAS_CONFIG, String.format(Utils.SASL_CREDENTIALS, username, password));

        properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, brokers);
        properties.put("basic.auth.credentials.source", "USER_INFO");
        properties.put("schema.registry.basic.auth.user.info", username + ":" + password);
        properties.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, "SASL_SSL");
        properties.put(SaslConfigs.SASL_MECHANISM, "SCRAM-SHA-512");
        properties.put("group.id", groupId);
        properties.put("auto.commit.enable", "false");
        properties.put("auto.offset.reset", "earliest");
        properties.put(ConsumerConfig.SESSION_TIMEOUT_MS_CONFIG, "20000");

        // avro part (deserializer)
        properties.setProperty("key.deserializer", StringDeserializer.class.getName());
        properties.setProperty("value.deserializer", SoftKafkaAvroDeserializer.class.getName());
        properties.setProperty("schema.registry.url", "https://schema-registry-security.dev.nautilus.app");
        properties.setProperty("specific.avro.reader", "false");

        KafkaConsumer<String, GenericRecord> kafkaConsumer = new KafkaConsumer<>(properties);
        kafkaConsumer.subscribe(Collections.singleton(topicName));

        System.out.println("Waiting for data on topic: " + topicName + "...");
        try{
            while (true){
                System.out.println("Polling");
                ConsumerRecords<String, GenericRecord> records = kafkaConsumer.poll(Duration.ofMillis(20000));
                for (ConsumerRecord<String, GenericRecord> record : records){
                    System.out.printf("offset = %d, key = %s, value = %s \n", record.offset(), record.key(), record.value());
                }
                kafkaConsumer.commitSync();
            }
        }catch (Exception e){
            System.out.printf("Error consuming message: %s \n", e.getMessage());
        }finally {
            kafkaConsumer.close();
        }
    }
}
