package kafka.example;

import org.apache.avro.Schema;
import org.apache.avro.generic.GenericData;
import org.apache.avro.generic.GenericRecord;
import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.config.SaslConfigs;
import org.apache.kafka.common.config.SslConfigs;
import kafka.example.config.SoftKafkaAvroSerializer;

import java.util.Properties;

public class Producer {

    public static void startProducer() {
        Properties props = new Properties();

        String username = "username";
        String password = "password";
        String topicName = "test-topic";
        String brokers = "public-broker1.nautilus.app:8096,public-broker2.nautilus.app:8096,public-broker3.nautilus.app:8096";

        props.put(SaslConfigs.SASL_JAAS_CONFIG, String.format(Utils.SASL_CREDENTIALS, username, password));

        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, brokers);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG,
                org.apache.kafka.common.serialization.StringSerializer.class);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, SoftKafkaAvroSerializer.class.getName());
        props.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, "SASL_SSL");
        props.put(SaslConfigs.SASL_MECHANISM, "SCRAM-SHA-512");
        props.put("schema.registry.url", "https://schema-registry-security.dev.nautilus.app");
        props.put("basic.auth.credentials.source", "USER_INFO");
        props.put("schema.registry.basic.auth.user.info", username + ":" + password);

        KafkaProducer producer = new KafkaProducer(props);
        String key = null; // Si no es necesrio se puede poner a null
        // Esquema Avro, para serialización y deserialización de mensajes.
        // Indica el formato del mensaje, esta en formato json escape
        String userSchema = "{\"type\":\"record\",\"name\":\"test\",\"namespace\":\"nautilus.test\",\"doc\":\"Test event schema\",\"fields\":[{\"name\":\"country\",\"type\":\"string\"},{\"name\":\"city\",\"type\":[\"null\",\"string\"],\"default\":null}]}";
        Schema.Parser parser = new Schema.Parser();
        Schema schema = parser.parse(userSchema);
        GenericRecord avroRecord = new GenericData.Record(schema);
        avroRecord.put("country", "France");
        avroRecord.put("city", "Paris");

        ProducerRecord<Object, Object> record = new ProducerRecord<>(topicName, key, avroRecord);
        try {
            producer.send(record);
            System.out.println("Message produced");
            System.out.println(record);
        } catch(Exception e) {
            System.out.printf("Error producing message: %s \n", e.getMessage());
        }
        finally {
            producer.flush();
            producer.close();
        }
    }
}
